@@ -1,5 +1,5 @@
 package Finance::Shares::moving_average;
-our $VERSION = 1.01;
+our $VERSION = 1.03;
 use strict;
 use warnings;
 use Finance::Shares::Support qw(%period out show);
@@ -87,7 +87,7 @@ sub build {
 	
 	push @points, $res;
     }
-    my $l = $o->line('mov');
+    my $l = $o->func_line('mov');
     $l->{data} = \@points;
 
     unless ($l->{key}) {
@@ -178,7 +178,7 @@ default)
 
 Required, unless B<graph> is given.  This specifies the type of graph the function
 lines should appear on.  It should be one of C<price>, C<volume>, C<analysis> or
-C<level>.  (Default: C<price>)
+C<logic>.  (Default: C<price>)
 
 =head3 line
 
