@@ -1,5 +1,5 @@
 package Finance::Shares::multiline_mean;
-our $VERSION = 1.01;
+our $VERSION = 1.03;
 use strict;
 use warnings;
 use Log::Agent;
@@ -32,7 +32,7 @@ sub build {
     my $o = shift;
     my $q = $o->{quotes};
     my $d = $q->dates;
-    my $l = $o->line('mean');
+    my $l = $o->func_line('mean');
 
     my @lines;
     foreach my $ar (@{$o->{line}}) {
@@ -130,7 +130,7 @@ default)
 
 Required, unless B<graph> is given.  This specifies the type of graph the function
 lines should appear on.  It should be one of C<price>, C<volume>, C<analysis> or
-C<level>.  (Default: C<price>)
+C<logic>.  (Default: C<price>)
 
 =head3 lines
 
